@@ -11,27 +11,30 @@ shiny_drive_ui <- function(id){
     singleton(tags$head(
       tags$script(src = "shinydrive/shiny_utils_sfm.js")
     )),
-
+    
     # fix fontAwesome init loading...
     fluidRow(
-      actionButton("fix FA", "fix FA", icon = icon("refresh"), style = "display:none"),
+      actionButton("fix FA", "fix FA", icon = icon("refresh"), style = "display:none")
     ),
-
-    fluidRow(
-      column(12,
-             fluidRow(
-               column(2,
-                      uiOutput(ns("ui_title"))
-               ),
-               column(4,
-                      style = "margin-left: 15px;",
-                      selectInput(ns("select_file_dir"), NULL, choices = NULL, selected = NULL, width = "100%")
-               ),
-               conditionalPanel("output.is_admin", ns = ns,
-                                uiOutput(ns("admin_dir_btn"))
-               )
-             )
-      )
+    
+    conditionalPanel(condition = "output.show_dir", ns = ns,
+                     fluidRow(
+                       column(12,
+                              fluidRow(
+                                column(2,
+                                       uiOutput(ns("ui_title"))
+                                ),
+                                column(4,
+                                       style = "margin-left: 15px;",
+                                       selectInput(ns("select_file_dir"), NULL, choices = NULL, selected = NULL, width = "100%")
+                                ),
+                                conditionalPanel("output.is_admin", ns = ns,
+                                                 column(5, uiOutput(ns("admin_dir_btn"))
+                                                 )
+                                )
+                              )
+                       )
+                     ) 
     ),
     # End fold management
     conditionalPanel("output.is_admin", ns = ns,
@@ -59,13 +62,16 @@ shiny_drive_ui <- function(id){
 #' @param input shiny input
 #' @param output shiny input
 #' @param session shiny input
-#' @param save_dir \code{character}. Main directory of the files.
+#' @param save_dir \code{character/reactive}. Main directory of the files.
 #' @param admin_user \code{boolean/reactive} (TRUE). Admin user or not.
 #' @param lan \code{character/reactive} ("EN"). Language to be used in the module (FR, EN and CN availabled... contributions are welcome :)).
-#' @param dir_access \code{character} vector for dir(s) access. Default to \code{NULL} (all directories)
+#' @param dir_access \code{character/reactive} vector for dir(s) access. Default to \code{NULL} (all directories)
 #' @param file_translate \code{data.frame/reactive} File for translation.
 #' @param force_desc \code{boolean/reactive} (FALSE). Force to add an entry description ?
-#'
+#' @param datatable_options \code{list/reactive}.  \code{DT::datatable} options argument.
+#' @param yml \code{characte/reactiver} yaml configuration file name.
+#' @param date_time_format \code{character} DateTime format.
+#' 
 #' @return Shiny module without return value.
 #' 
 #' @importFrom utils zip read.csv
@@ -108,73 +114,105 @@ shiny_drive_server <- function(input,
                                file_translate = read.csv(system.file("translate/translate.csv", package = "shinydrive"),
                                                          sep = ";",
                                                          encoding = "UTF-8",
-                                                         check.names=FALSE)) {
-
+                                                         check.names=FALSE), 
+                               datatable_options = list(), 
+                               yml = "files_desc.yaml", 
+                               date_time_format = "%Y%m%d_%H%M%s") {
+  
   ns <- session$ns
-
+  
+  if (!shiny::is.reactive(yml)){
+    get_yml <- shiny::reactive({yml})
+  } else {
+    get_yml <- yml
+  }
+  
   if (!shiny::is.reactive(save_dir)){
     get_save_dir <- shiny::reactive({save_dir})
   } else {
     get_save_dir <- save_dir
   }
-
+  
   if (!shiny::is.reactive(dir_access)){
     get_dir_access <- shiny::reactive({dir_access})
   } else {
     get_dir_access <- dir_access
   }
-
+  
   if (!shiny::is.reactive(admin_user)){
     get_admin_user <- shiny::reactive({admin_user})
   } else {
     get_admin_user <- admin_user
   }
-
+  
   if (!shiny::is.reactive(lan)){
     get_lan <- shiny::reactive({lan})
   } else {
     get_lan <- lan
   }
-
+  
   if (!shiny::is.reactive(file_translate)){
     get_file_translate <- shiny::reactive({file_translate})
   } else {
     get_file_translate <- file_translate
   }
-
+  
   if (!shiny::is.reactive(force_desc)){
     get_force_desc <- shiny::reactive({force_desc})
   } else {
     get_force_desc <- force_desc
   }
-
+  if (! shiny::is.reactive(datatable_options)) {
+    get_datatable_options <- shiny::reactive(datatable_options)
+  } else {
+    get_datatable_options <- datatable_options
+  }
+  
   output$msg_no_file <- renderUI({
     file_translate <- get_file_translate()
     req(file_translate)
     div(h4(file_translate[file_translate$ID == 45, get_lan()]), align = "center")
   })
-
+  
   # Admin TRUE (pass to module at final version)
   output$is_admin <- reactive({
     get_admin_user()
   })
   outputOptions(output, "is_admin", suspendWhenHidden = FALSE)
-
+  
+  output$show_dir <- reactive({
+    if(is.null(get_admin_user())){
+      FALSE
+    } else {
+      (get_admin_user() | (!get_admin_user() & length(list.available.dirs()) >= 1))
+    }
+  })
+  outputOptions(output, "show_dir", suspendWhenHidden = FALSE)
+  
   output$ui_title <- renderUI({
     file_translate <- get_file_translate()
     req(file_translate)
     div(h4(file_translate[file_translate$ID == 1, get_lan()]), align = "center")
   })
-
+  
   output$admin_dir_btn <- renderUI({
     file_translate <- get_file_translate()
     req(file_translate)
-    column(5,
-           actionButton(ns("create_file_dir_bis"),file_translate[file_translate$ID == 2, get_lan()], icon = icon("plus")),
-           actionButton(ns("rename_file_dir"), file_translate[file_translate$ID == 3, get_lan()], icon = icon("edit"))
+    fluidRow(
+      column(4, actionButton(ns("create_file_dir_bis"),file_translate[file_translate$ID == 2, get_lan()], icon = icon("plus"))),
+      column(4, 
+             conditionalPanel("input.select_file_dir !== '/'", ns = ns,
+                              actionButton(ns("rename_file_dir"), file_translate[file_translate$ID == 3, get_lan()], icon = icon("edit"))
+             )
+      ),
+      column(4, 
+             conditionalPanel("input.select_file_dir !== '/'", ns = ns,
+                              actionButton(ns("remove_file_dir"), file_translate[file_translate$ID == 47, get_lan()], icon = icon("trash"))
+             )
+      )
     )
   })
-
+  
   output$admin_add_file <- renderUI({
     file_translate <- get_file_translate()
     req(file_translate)
@@ -187,24 +225,34 @@ shiny_drive_server <- function(input,
     )
   })
   # gestion dossier
-  list.available.dirs <- reactiveFileReader(1000, session, get_save_dir,
-                                            function(x){
-                                              if (!is.null(x) && length(x) > 0 && dir.exists(x)){
-                                                list.dirs(x, recursive = F, full.names = F)
-                                              } else {
-                                                NULL
-                                              }
-                                            })
-
-
+  auto_udpate <- reactiveTimer(1000)
+  list.available.dirs <- reactiveVal(NULL)
+  
+  observe({
+    get_save_dir <- get_save_dir()
+    req(get_save_dir)
+    auto_udpate()
+    if (!is.null(get_save_dir) && length(get_save_dir) > 0 && dir.exists(get_save_dir)){
+      val <- list.dirs(get_save_dir, recursive = T, full.names = F)
+    } else {
+      val <- NULL
+    }
+    if(!isTRUE(all.equal(val, isolate(list.available.dirs())))){
+      list.available.dirs(val)
+    }
+  })
+  
+  
   current_dir <- reactiveVal(NULL)
-
+  
   observe({
     select_file_dir <- input$select_file_dir
-    if(select_file_dir == "") select_file_dir <- "/"
-    current_dir(select_file_dir)
+    if(!is.null(select_file_dir)){
+      if(select_file_dir == "") select_file_dir <- "/"
+      current_dir(select_file_dir)
+    }
   })
-
+  
   observe({
     dir_access <- get_dir_access()
     list.available.dirs <- list.available.dirs()
@@ -213,10 +261,10 @@ shiny_drive_server <- function(input,
       if("" %in% dir_access)  dir_access <- c(dir_access, "/")
       list.available.dirs <- list.available.dirs[list.available.dirs %in% dir_access]
     }
-
+    
     isolate({
       if (!is.null(list.available.dirs) ){
-        if(current_dir() %in% list.available.dirs){
+        if(!is.null(current_dir()) && current_dir() %in% list.available.dirs){
           sel <- current_dir()
         } else {
           sel <- NULL
@@ -228,15 +276,15 @@ shiny_drive_server <- function(input,
       }
     })
   })
-
-
+  
+  
   observeEvent(input$create_file_dir_bis, {
     file_translate <- get_file_translate()
-
+    
     removeModal()
-
+    
     shiny::showModal(shiny::modalDialog(
-      title = div(file_translate[file_translate$ID == 5, get_lan()], style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
+      title = div(paste0(ifelse(input$select_file_dir != "/", paste0(input$select_file_dir, " : "), ""), file_translate[file_translate$ID == 5, get_lan()]), style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
       shiny::textInput(ns("file_dir_desc"), file_translate[file_translate$ID == 6, get_lan()], value="", width = "100%"),
       footer = fluidRow(
         column(6, div(actionButton(ns("create_file_dir_ok"), file_translate[file_translate$ID == 2, get_lan()]), align = "center")),
@@ -246,19 +294,21 @@ shiny_drive_server <- function(input,
       easyClose = FALSE
     ))
   }, ignoreInit = TRUE)
-
+  
   observeEvent(input$rename_file_dir,{
     file_translate <- get_file_translate()
-
+    
     list_dirs <- list.available.dirs()
-
+    
+    req(list_dirs)
+    
     if(length(list_dirs) > 0){
-
+      
       removeModal()
-
+      
       shiny::showModal(shiny::modalDialog(
-        title = div(file_translate[file_translate$ID == 39, get_lan()], style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
-        selectInput(ns("select_file_dir_rename"), file_translate[file_translate$ID == 1, get_lan()], choices = list_dirs),
+        title = div(paste0(ifelse(input$select_file_dir != "/", paste0(input$select_file_dir, " : "), ""), file_translate[file_translate$ID == 39, get_lan()]), style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
+        # selectInput(ns("select_file_dir_rename"), file_translate[file_translate$ID == 1, get_lan()], choices = list_dirs),
         shiny::textInput(ns("new_file_dir_desc"),  file_translate[file_translate$ID == 40, get_lan()],
                          value = "", width = "100%"),
         footer = fluidRow(
@@ -269,9 +319,9 @@ shiny_drive_server <- function(input,
         easyClose = FALSE
       ))
     } else {
-
+      
       removeModal()
-
+      
       shiny::showModal(shiny::modalDialog(
         easyClose = TRUE,
         footer = NULL,
@@ -279,21 +329,22 @@ shiny_drive_server <- function(input,
       ))
     }
   }, ignoreInit = TRUE)
-
+  
+  
   # Renomer un sous-dossier
   observeEvent(input$rename_file_dir_selected,{
-
+    
     save_dir <- isolate(get_save_dir())
-
+    
     file_translate <- get_file_translate()
-
+    
     list_dirs <- list.available.dirs()
-
-    if (input$new_file_dir_desc==""){
+    
+    if (!is.null(input$new_file_dir_desc) && input$new_file_dir_desc == ""){
       removeModal()
-
+      
       shiny::showModal(shiny::modalDialog(
-        title = div(file_translate[file_translate$ID == 39, get_lan()], style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
+        title = div(paste0(ifelse(input$select_file_dir != "/", paste0(input$select_file_dir, " : "), ""), file_translate[file_translate$ID == 39, get_lan()]), style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
         shiny::textInput(ns("new_file_dir_desc"), file_translate[file_translate$ID == 40, get_lan()],
                          value = "", width = "100%"),
         easyClose = FALSE,
@@ -303,42 +354,86 @@ shiny_drive_server <- function(input,
           )
         )
       ))
-    } else if (input$new_file_dir_desc %in% list_dirs){
-      removeModal()
+    } else {
 
+      new_dir <- strsplit(input$select_file_dir, "/")[[1]]
+      new_dir <- file.path(paste0(new_dir[-length(new_dir)], collapse = "/"), input$new_file_dir_desc)
+      
+      if (new_dir %in% list_dirs){
+        removeModal()
+        
+        shiny::showModal(shiny::modalDialog(
+          title = div(paste0(ifelse(input$select_file_dir != "/", paste0(input$select_file_dir, " : "), ""), file_translate[file_translate$ID == 39, get_lan()]), style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
+          shiny::textInput(ns("new_file_dir_desc"), file_translate[file_translate$ID == 8, get_lan()],
+                           value = "", width = "100%"),
+          easyClose = FALSE,
+          footer = fluidRow(
+            column(6, div(actionButton(ns("rename_file_dir_selected"),  file_translate[file_translate$ID == 3, get_lan()]), align = "center")),
+            column(6, div(modalButton(file_translate[file_translate$ID == 7, get_lan()]), align = "center")
+            )
+          )
+        ))
+      } else {
+        file.rename(from = file.path(save_dir, input$select_file_dir),
+                    to = file.path(save_dir, new_dir))
+        removeModal()
+        shiny::showModal(shiny::modalDialog(
+          easyClose = TRUE,
+          footer = NULL,
+          file_translate[file_translate$ID == 41, get_lan()]
+        ))
+      }
+    }
+  }, ignoreInit = TRUE)
+  
+  # supprimer un dossier
+  observeEvent(input$remove_file_dir,{
+    file_translate <- get_file_translate()
+    
+    list_dirs <- list.available.dirs()
+    
+    req(list_dirs)
+    req(input$select_file_dir)
+    
+    if(length(list_dirs) > 0 && !input$select_file_dir %in% c("", "/")){
+      
+      removeModal()
+      
       shiny::showModal(shiny::modalDialog(
-        title = div(file_translate[file_translate$ID == 39, get_lan()], style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
-        shiny::textInput(ns("new_file_dir_desc"), file_translate[file_translate$ID == 8, get_lan()],
-                         value = "", width = "100%"),
-        easyClose = FALSE,
+        title = div(file_translate[file_translate$ID == 42, get_lan()], style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
+        div(h4(input$select_file_dir), align = "center"),
         footer = fluidRow(
-          column(6, div(actionButton(ns("rename_file_dir_selected"),  file_translate[file_translate$ID == 3, get_lan()]), align = "center")),
+          column(6, div(actionButton(ns("remove_file_dir_ok"), file_translate[file_translate$ID == 47, get_lan()]), align = "center")),
           column(6, div(modalButton(file_translate[file_translate$ID == 7, get_lan()]), align = "center")
           )
-        )
-      ))
-    } else {
-      file.rename(from = file.path(save_dir, input$select_file_dir_rename),
-                  to = file.path(save_dir, input$new_file_dir_desc))
-      removeModal()
-      shiny::showModal(shiny::modalDialog(
-        easyClose = TRUE,
-        footer = NULL,
-        file_translate[file_translate$ID == 41, get_lan()]
+        ),
+        easyClose = FALSE
       ))
     }
   }, ignoreInit = TRUE)
-
-  observeEvent(input$create_file_dir_ok,{
-
+  
+  observeEvent(input$remove_file_dir_ok,{
     save_dir <- isolate(get_save_dir())
-
+    req(save_dir)
+    req(input$select_file_dir)
+    if(!input$select_file_dir %in% c("", "/")){
+      tryCatch({
+        fold_path <- file.path(save_dir, input$select_file_dir)
+        unlink(fold_path, recursive = TRUE)
+      }, error = function(e) NULL)
+    }
+    removeModal()
+  }, ignoreInit = TRUE)
+  
+  observeEvent(input$create_file_dir_ok,{
+    
+    save_dir <- isolate(get_save_dir())
+    
     file_translate <- get_file_translate()
-
-    if (input$file_dir_desc == ""){
-      # Donner un description au file
+    
+    if (!is.null(input$file_dir_desc) && input$file_dir_desc == ""){
       shiny::showModal(shiny::modalDialog(
-        title = div(file_translate[file_translate$ID == 5, get_lan()], style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
+        title = div(paste0(ifelse(input$select_file_dir != "/", paste0(input$select_file_dir, " : "), ""), file_translate[file_translate$ID == 5, get_lan()]), style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
         shiny::textInput(ns("file_dir_desc"), file_translate[file_translate$ID == 6, get_lan()], value="", width = "100%"),
         footer = fluidRow(
           column(6, div(actionButton(ns("create_file_dir_ok"), file_translate[file_translate$ID == 2, get_lan()]), align = "center")),
@@ -348,9 +443,16 @@ shiny_drive_server <- function(input,
         easyClose = FALSE
       ))
     } else {
-      if (input$file_dir_desc %in% list.available.dirs()){
+      
+      if(isolate(input$select_file_dir) != "/"){
+        new_dir <- gsub("^/", "", file.path(isolate(input$select_file_dir), input$file_dir_desc))
+      } else {
+        new_dir <- input$file_dir_desc
+      }
+      
+      if (new_dir %in% list.available.dirs()){
         removeModal()
-
+        
         shiny::showModal(shiny::modalDialog(
           title = div(file_translate[file_translate$ID == 5, get_lan()], style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
           shiny::textInput(ns("file_dir_desc"), file_translate[file_translate$ID == 8, get_lan()], value="", width = "100%"),
@@ -362,18 +464,18 @@ shiny_drive_server <- function(input,
           easyClose = FALSE
         ))
       } else {
-        fold_path <- file.path(save_dir, input$file_dir_desc)
+        fold_path <- file.path(save_dir, new_dir)
         create_test <- dir.create(fold_path)
         if (create_test){
-          file.create(file.path(fold_path, "files_desc.yaml"))
-          current_dir(input$file_dir_desc)
+          file.create(file.path(fold_path, isolate(get_yml())))
+          current_dir(new_dir)
           removeModal()
           shiny::showModal(shiny::modalDialog(
             easyClose = TRUE,
             footer = NULL,
             file_translate[file_translate$ID == 9, get_lan()]
           ))
-
+          
         } else {
           removeModal()
           shiny::showModal(shiny::modalDialog(
@@ -385,53 +487,58 @@ shiny_drive_server <- function(input,
       }
     }
   }, ignoreInit = TRUE)
-
-  yml <- reactive({
-
+  
+  req_yml <- reactive({
+    
     save_dir <- get_save_dir()
-
+    
     if(!is.null(input$select_file_dir) && input$select_file_dir != ""){
       if(input$select_file_dir !="/"){
-        file.path(save_dir, input$select_file_dir, "files_desc.yaml")
+        file.path(save_dir, input$select_file_dir, isolate(get_yml()))
       }else{
-        file.path(save_dir, "files_desc.yaml")
+        file.path(save_dir, isolate(get_yml()))
       }
     } else {
       ""
     }
   })
-
+  
   # End gestion dossier
-
-  all_files <- reactiveFileReader(1000, session, yml, function(x){
+  
+  all_files <- reactiveFileReader(1000, session, req_yml, function(x){
     if (!is.null(x) && length(x) > 0 && file.exists(x)){
-      .yaml_to_dt(x)
+      get_yaml_info(x, 
+                    recorded_name = TRUE,
+                    date_time_format = date_time_format, 
+                    add_img = TRUE, 
+                    img_size = 30
+      )
     } else {
       NULL
     }})
-
+  
   output$have_files <- reactive({
     !is.null(all_files()) && nrow(all_files()) > 0
   })
   outputOptions(output, "have_files", suspendWhenHidden = FALSE)
-
+  
   # launch modal to add a new file
   count_file_load <- reactiveVal(round(runif(1, 1, 100000000), 0))
-
+  
   output$file_load <- renderUI({
     file_translate <- get_file_translate()
-
+    
     fluidRow(
       column(12,
              fileInput(ns(paste0("file_load", count_file_load())), label = file_translate[file_translate$ID == 11, get_lan()])
       )
     )
   })
-
+  
   output$file_comp_load <- renderUI({
     file_name <- input[[paste0("file_load", count_file_load())]]$name
     file_translate <- get_file_translate()
-
+    
     if(is.null(file_name)){
       fluidRow()
     } else {
@@ -444,14 +551,14 @@ shiny_drive_server <- function(input,
       )
     }
   })
-
+  
   observeEvent(input$add_file, {
     file_translate <- get_file_translate()
-
+    
     count_file_load(count_file_load() + 1)
-
+    
     removeModal()
-
+    
     showModal(modalDialog(
       title = div(file_translate[file_translate$ID == 4, get_lan()], style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
       easyClose = F,
@@ -469,84 +576,103 @@ shiny_drive_server <- function(input,
       )
     ))
   }, ignoreInit = TRUE)
-
+  
   observe({
     file_info <- input[[paste0("file_load", count_file_load())]]
     name <- input$file_name
     description <- input$description
-    req(get_force_desc())
-    if (isolate(get_force_desc()) && length(file_info) > 0 && length(name) > 0 && name != "" && length(description) > 0 && description != "") {
-      toggleBtn(session = session, inputId = ns("added_file"), type = "enable")
-    } else if (!isolate(get_force_desc()) && length(file_info) > 0 && length(name) > 0  && name != "") {
-      toggleBtn(session = session, inputId = ns("added_file"), type = "enable")
-    } else {
-      toggleBtn(session = session, inputId = ns("added_file"), type = "disable")
+    
+    if(!is.null(isolate(get_force_desc()))){
+      if (isolate(get_force_desc()) && length(file_info) > 0 && length(name) > 0 && name != "" && length(description) > 0 && description != "") {
+        toggleBtn(session = session, inputId = ns("added_file"), type = "enable")
+      } else if (!isolate(get_force_desc()) && length(file_info) > 0 && length(name) > 0  && name != "") {
+        toggleBtn(session = session, inputId = ns("added_file"), type = "enable")
+      } else {
+        toggleBtn(session = session, inputId = ns("added_file"), type = "disable")
+      }
     }
   })
-
+  
   # Add a file
   observeEvent(input$added_file, {
-
+    
     save_dir <- isolate(get_save_dir())
-
+    
     file_info <- input[[paste0("file_load", count_file_load())]]
-
+    
     # To folder
-    if(input$select_file_dir != "/"){
+    if(!is.null(input$select_file_dir) && input$select_file_dir != "/"){
       dir <- file.path(save_dir, input$select_file_dir)
     }else{
       dir <- save_dir
     }
-
-    add_file_in_dir(
-      file = file_info$datapath,
-      dir = dir,
-      name = input$file_name,
-      yml = yml(),
-      description = input$description
-    )
-
+    
+    
     removeModal()
-
+    
+    ctrl_add <- tryCatch({
+      add_file_in_dir(
+        file = file_info$datapath,
+        dir = dir,
+        name = input$file_name,
+        yml = req_yml(),
+        description = input$description, 
+        date_time_format = date_time_format
+      )
+    },
+    error = function(e){
+      showModal(
+        modalDialog(
+          title = "Error adding file",
+          easyClose = TRUE,
+          footer = NULL,
+          e$message
+        ))
+      NULL
+    })
+    
   }, ignoreInit = TRUE)
-
+  
   uniquenames <- reactive({
     req(all_files())
     dt <- all_files()
-    uniquenames <- paste0(tools::file_path_sans_ext(dt$name), dt$date_time)
+    uniquenames <- gsub("([[:space:]])+|([[:punct:]])+", "_", paste0(tools::file_path_sans_ext(dt$name), dt$date_time))
     uniquenames
   })
-
+  
   ctname <- reactive({
     all_files()
     paste0(sample(LETTERS, 5), collapse = "")
   })
-
+  
   output$dt <- DT::renderDT({
-
+    
     unbindDTSFM(ns("dt"))
-
+    
     req(all_files())
+    
     dt <- all_files()
+    dt$recorded_name <- NULL
+    
     file_translate <- get_file_translate()
-
+    
     if(nrow(dt) == 0) return(NULL)
-
-    if(get_admin_user()){
+    
+    if(!is.null(get_admin_user) && get_admin_user()){
       dt$Edit <- input_btns(ns("edit_file"), uniquenames(), file_translate[file_translate$ID == 16, get_lan()], icon("pencil-square-o"), status = "primary")
       dt$Remove <- input_btns(ns("remove_file"), uniquenames(), file_translate[file_translate$ID == 17, get_lan()], icon("trash-o"), status = "danger")
     }
-
+    
     dt$Download <- input_btns(ns("download_file"), uniquenames(), file_translate[file_translate$ID == 18, get_lan()], icon("download"), status = "success")
-
-    dt$Select <- input_checkbox_ui(ns("remove_mult_files"), paste0(uniquenames(), ctname()), checked = FALSE)
-
+    
+    dt$Select <- input_checkbox_ui(ns("remove_mult_files"), paste0(uniquenames(), ctname()), 
+                                   session = session, 
+                                   checked = FALSE)
+    
     file_translate[[get_lan()]] <- as.character(file_translate[[get_lan()]])
-
-    dt$date_time <- format(as.POSIXct(as.character(dt$date_time), format = "%Y%m%d_%H%M%s"))
-
+    
     dt$id <- NULL
-
+    
     if(get_admin_user()){
       if(ncol(dt) < 8){return(NULL)}
       names(dt) <- c(file_translate[file_translate$ID == 46, get_lan()],
@@ -562,7 +688,7 @@ shiny_drive_server <- function(input,
       target_wd_cols <- c(0, (ncol(dt)-4):(ncol(dt)-1))
     }else{
       if(ncol(dt) < 6){return(NULL)}
-
+      
       names(dt) <- c( file_translate[file_translate$ID == 46, get_lan()],
                       file_translate[file_translate$ID == 19, get_lan()],
                       file_translate[file_translate$ID == 20, get_lan()],
@@ -572,7 +698,24 @@ shiny_drive_server <- function(input,
       
       target_wd_cols <- c(0, (ncol(dt)-2):(ncol(dt)-1))
     }
-
+    
+    default_options = list(
+      language = list(url = file_translate[file_translate$ID == 26, get_lan()]),
+      drawCallback = DT::JS("function() {Shiny.bindAll(this.api().table().node());}"),
+      scrollX = TRUE,
+      columnDefs = list(
+        list(className =  "dt-head-center", "targets" = "_all"),
+        list(width = "50px", targets = target_wd_cols)
+      )
+    )
+    
+    custom_options <- get_datatable_options()
+    if(length(custom_options) > 0){
+      for(n in names(custom_options)){
+        default_options[[n]] <- custom_options[[n]]
+      }
+    }
+    
     DT::datatable(
       data = dt,
       colnames = make_title(names(dt)),
@@ -581,49 +724,36 @@ shiny_drive_server <- function(input,
       selection = "none",
       extensions = 'AutoFill',
       #extensions = 'FixedColumns', # bug using FixedColumns on checkbox + update table...
-      options = list(
-        language = list(url = file_translate[file_translate$ID == 26, get_lan()]),
-        drawCallback = DT::JS("function() {Shiny.bindAll(this.api().table().node());}"),
-        scrollX = TRUE,
-        columnDefs = list(
-          list(width = "50px", targets = target_wd_cols)
-        )
-      )
+      options = default_options
     )
-  })
-
+  }, server = FALSE)
+  
   download_file_r <- reactive({
     dt <- all_files()
     all_names <- uniquenames()
     dt_sel <- dt[all_names %in% input$download_file, ]
     dt_sel
   })
-
+  
   download_file_rf <- reactive({
-
+    
     save_dir <- isolate(get_save_dir())
-
-    if(input$select_file_dir != "/"){
-      fp <- file.path(save_dir, input$select_file_dir,
-                      paste0(tools::file_path_sans_ext(download_file_r()$name),"_",
-                             download_file_r()$date_time, ".",
-                             tools::file_ext(download_file_r()$name)))
+    
+    if(!is.null(input$select_file_dir) && input$select_file_dir != "/"){
+      fp <- file.path(save_dir, input$select_file_dir, download_file_r()$recorded_name)
     }else{
-      fp <- file.path(save_dir,
-                      paste0(tools::file_path_sans_ext(download_file_r()$name),"_",
-                             download_file_r()$date_time, ".",
-                             tools::file_ext(download_file_r()$name)))
+      fp <- file.path(save_dir,download_file_r()$recorded_name)
     }
     fp
   })
-
-
-
+  
+  
+  
   observeEvent(input$download_file, {
     file_translate <- get_file_translate()
-
+    
     removeModal()
-
+    
     showModal(modalDialog(
       title =  div(file_translate[file_translate$ID == 18, get_lan()], style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
       ui_describ_file(tools::file_path_sans_ext(download_file_r()$name),
@@ -640,40 +770,40 @@ shiny_drive_server <- function(input,
       )
     ))
   }, ignoreInit = TRUE)
-
+  
   # download_file
   output$downloaded_file_sgl <- downloadHandler(
-
+    
     filename <- function() {
       paste0(tools::file_path_sans_ext(download_file_r()$name),  ".",
              tools::file_ext(download_file_r()$name))
-
+      
     },
-
+    
     content <- function(file) {
       fp <- download_file_rf()
       removeModal()
       file.copy(fp, file)
     }
   )
-
+  
   file_to_edit <- reactive({
     dt <- all_files()
     all_names <- uniquenames()
     dt_sel <- dt[all_names == input$edit_file, ]
     dt_sel
   })
-
-
-
+  
+  
+  
   # Edit file
   observeEvent(input$edit_file, {
     cpt <- count_file_load() + 1
     count_file_load(cpt)
     file_translate <- get_file_translate()
-
+    
     removeModal()
-
+    
     showModal(modalDialog(
       title = div(file_translate[file_translate$ID == 16, get_lan()], style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
       fluidRow(
@@ -704,12 +834,12 @@ shiny_drive_server <- function(input,
       )
     ))
   }, ignoreInit = TRUE)
-
+  
   observe({
     file_info <- input[[paste0("file_load", count_file_load())]]
     name <- input$file_name_bis
     description <- input$description_bis
-
+    
     if(!is.null(input$load_new)){
       if (input$load_new && isolate(get_force_desc()) && length(file_info) > 0 && length(name) > 0 && name != "" && length(description) > 0 && description != "") {
         toggleBtn(session = session, inputId = ns("edited_file"), type = "enable")
@@ -724,36 +854,50 @@ shiny_drive_server <- function(input,
       }
     }
   })
-
+  
   observeEvent(input$edited_file, {
-
+    
     save_dir <- isolate(get_save_dir())
-
+    
     if(!is.null(input$load_new)){
       file_info <- input[[paste0("file_load", count_file_load())]]
     } else {
       file_info <- NULL
     }
-
-    if(input$select_file_dir != "/"){
+    
+    if(!is.null(input$select_file_dir) && input$select_file_dir != "/"){
       dir <- file.path(save_dir, input$select_file_dir)
     }else{
       dir <- save_dir
     }
-
+    
     # Write yaml edited
-    edit_file_in_dir(id = as.character(file_to_edit()$id),
-                     dir = dir,
-                     yml = yml(),
-                     name = input$file_name_bis,
-                     description = input$description_bis,
-                     file = file_info$datapath)
-
     removeModal()
+    
+    ctrl_edit <- tryCatch({
+      edit_file_in_dir(id = as.character(file_to_edit()$id),
+                       dir = dir,
+                       yml = req_yml(),
+                       name = input$file_name_bis,
+                       description = input$description_bis,
+                       file = file_info$datapath, 
+                       date_time_format = date_time_format)
+    },
+    error = function(e){
+      showModal(
+        modalDialog(
+          title = "Error editing file",
+          easyClose = TRUE,
+          footer = NULL,
+          e$message
+        ))
+      NULL
+    })
+    
   }, ignoreInit = TRUE)
   # End edit file
-
-
+  
+  
   # Suppress file
   file_to_remove <- reactive({
     dt <- all_files()
@@ -761,12 +905,12 @@ shiny_drive_server <- function(input,
     dt_sel <- dt[all_names == input$remove_file, ]
     dt_sel
   })
-
+  
   observeEvent(input$remove_file, {
     file_translate <- get_file_translate()
-
+    
     removeModal()
-
+    
     showModal(modalDialog(
       title = div(file_translate[file_translate$ID == 17, get_lan()], style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
       ui_describ_file(tools::file_path_sans_ext(file_to_remove()$name),
@@ -786,31 +930,46 @@ shiny_drive_server <- function(input,
       )
     ))
   }, ignoreInit = TRUE)
-
+  
   observeEvent(input$removed_file, {
-
+    
     save_dir <- isolate(get_save_dir())
-
-    if(input$select_file_dir != "/"){
+    
+    if(!is.null(input$select_file_dir) && input$select_file_dir != "/"){
       dir <- file.path(save_dir, input$select_file_dir)
     }else{
       dir <- save_dir
     }
-
-    suppress_file_in_dir(id = as.character(file_to_remove()$id),
-                         dir = dir,
-                         yml = yml())
+    
+    
+    ctrl_rm <- tryCatch({
+      suppress_file_in_dir(id = as.character(file_to_remove()$id),
+                           dir = dir,
+                           yml = req_yml())
+    },
+    error = function(e){
+      showModal(
+        modalDialog(
+          title = "Error removing file",
+          easyClose = TRUE,
+          footer = NULL,
+          e$message
+        ))
+      NULL
+    })
+    
   }, ignoreInit = TRUE)
-
+  
   # Remove multiple
   r_selected_files <- callModule(module = input_checkbox, id = "remove_mult_files")
-
-
+  
   # # # Remove all selected files
   output$supress_all <- renderUI({
     r_selected_files()
     file_translate <- get_file_translate()
-
+    
+    req(files_to_remove())
+    
     if(nrow(files_to_remove()) > 1){
       div(
         conditionalPanel("output.is_admin",ns = ns,
@@ -829,24 +988,24 @@ shiny_drive_server <- function(input,
       )
     }
   })
-
-
+  
+  
   all_output <- reactive({
     selected_files <- r_selected_files()
     selected_files
   })
-
+  
   files_to_remove <- reactive({
     req(all_output())
     dt_sel <- all_files()[paste0(uniquenames(), ctname()) %in%  all_output(), ]
     dt_sel
   })
-
+  
   observeEvent(input$remove_selected_files, {
     file_translate <- get_file_translate()
-
+    
     removeModal()
-
+    
     showModal(
       modalDialog(
         title = div(file_translate[file_translate$ID == 31, get_lan()], style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
@@ -871,32 +1030,44 @@ shiny_drive_server <- function(input,
         )
       ))
   }, ignoreInit = TRUE)
-
-
+  
+  
   observeEvent(input$removed_selected_files, {
-
+    
     save_dir <- isolate(get_save_dir())
-
-    if(input$select_file_dir != "/"){
+    
+    if(!is.null(input$select_file_dir) && input$select_file_dir != "/"){
       dir <- file.path(save_dir, input$select_file_dir)
     }else{
       dir <- save_dir
     }
-
+    
     for(i in 1:nrow(files_to_remove())){
-      suppress_file_in_dir(id = as.character(files_to_remove()[i, "id"]),
-                           dir = dir,
-                           yml = yml())
-
+      
+      ctrl_rm <- tryCatch({
+        suppress_file_in_dir(id = as.character(files_to_remove()[i, "id"]),
+                             dir = dir,
+                             yml = req_yml())
+      },
+      error = function(e){
+        showModal(
+          modalDialog(
+            title = "Error removing file",
+            easyClose = TRUE,
+            footer = NULL,
+            e$message
+          ))
+        NULL
+      })
     }
   }, ignoreInit = TRUE)
-
+  
   #  And download all files
   observeEvent(input$download_delected_files, {
     file_translate <- get_file_translate()
-
+    
     removeModal()
-
+    
     showModal(modalDialog(
       title = div(file_translate[file_translate$ID == 34, get_lan()], style = "color: #337ab7; font-size: 25px; font-weight: bold;"),
       tags$div(id = "placeholder-editfile-exist"),
@@ -919,32 +1090,26 @@ shiny_drive_server <- function(input,
       )
     ))
   }, ignoreInit = TRUE)
-
+  
   download_all_file_rf <- reactive({
-
+    
     save_dir <- isolate(get_save_dir())
-
+    
     sapply(1:nrow(files_to_remove()), function(i){
       download_file_r <- files_to_remove()[i, ]
       if((input$select_file_dir != "/")){
-        fp <- file.path(save_dir, input$select_file_dir,
-                        paste0(tools::file_path_sans_ext(download_file_r$name),"_",
-                               download_file_r$date_time, ".",
-                               tools::file_ext(download_file_r$name)))
+        fp <- file.path(save_dir, input$select_file_dir, download_file_r$recorded_name)
         names(fp) <- paste0(tools::file_path_sans_ext(download_file_r$name),  ".",
                             tools::file_ext(download_file_r$name))
       }else{
-        fp <- file.path(save_dir,
-                        paste0(tools::file_path_sans_ext(download_file_r$name),"_",
-                               download_file_r$date_time, ".",
-                               tools::file_ext(download_file_r$name)))
+        fp <- file.path(save_dir, download_file_r$recorded_name)
         names(fp) <- paste0(tools::file_path_sans_ext(download_file_r$name),  ".",
                             tools::file_ext(download_file_r$name))
       }
       fp
     })
   })
-
+  
   output$downloaded_file <- downloadHandler(
     filename <- function() {
       paste0("shinydrive_files_", format(Sys.time(), format = "%Y%m%d_%H%M%S"), ".zip")
@@ -959,8 +1124,8 @@ shiny_drive_server <- function(input,
       })
       removeModal()
       zip(file, tmp_fp, flags = "-r9X -j")
-
-      tryCatch({file.remove(tmp_fp)}, error = function(e) NULL)
+      
+      tryCatch({file.remove(tmp_fp)}, error = function(e) NULL, warning = function(e) NULL)
     }
   )
 }
